;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;
;   function_counts2photons
;
; PURPOSE:
;
;   Inversion of count spectrum using regularisation + GSVD method
;   to obtain model-independent photon spectrum
;
;   Quick start with the inversion software
;   http://www.astro.gla.ac.uk/users/eduard/rhessi/inversion/
;
;
;  The observed count flux spectrum in the X-ray detectors
;  C(eps) [counts/sec/cm^2/keV] can be written in matrix form
;
;  C = DRM*F
;
;  DRM - dynamic responce matrix of instrument
;
;  We know C and DRM and want to find a photon flux F  [counts/sec/cm^2/keV].
;  This problem is ill-conditioned
;  and regularision (additional constraints) should be applied.
;
;  The code finds the solution for the problem as a function of lambda
;
;  ||DRM*F-C||^2+lambda*||L*(F-F0)||^2=min
;
;   where
;
;   ||...|| - L2 norm
;   L       - constraint matrix
;   F0      - a model of photon spectrum from SPEX/OSPEX
;   lambda  - regularisation parameter found using descripancy principle
;
;
;   ||A*F(lambda)-C||=reg_tweak*||dC||
;
;   where
;   dC  - 1 sigma uncertanty on count flux, [counts/sec/cm^2/keV]
;
;   For details see:
;   Kontar et al,2004,Sol Phys,225,293-309
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2004SoPh..225..293K)
;   Kontar et al,2005,Sol Phys,226,317-325
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2005SoPh..226..317K)
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
; function_COUNTS2PHOTONS,e_c2N,eph2N,drm,counts,ecounts,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak,data_id=data_id,silent=silent
;
; CALLS:
;
;
; INPUTS:
;
; e_c2N  - 2D array, energy edges for counts, keV
; eph2N  - 2D array, energy edges for photons, keV
; drm    - drm matrix, [counts /sec/cm^2/keV /(photons /sec/cm^2/keV)]
; counts - count flux,       [counts /sec/cm^2/keV]
; ecounts- count flux errors,[counts /sec/cm^2/keV]
;
;
; OPTIONAL INPUTS:
;
;   version   - string, version of the inversion code (currently only one version exist)
;   order     - integer, regularization order. Can be 0 (default), 1, or 2.
;   Npass     - integer, number of random noise realisations for the confidence strip.
;                 Default is 30.
;   Guess     - integer, 1 or 0 (default) - to use or NOT to use forward-fit in the constraint
;   reg_tweak - float,  parameter to adjust the regularization parameter (default is 1).
;   data_id   - string, to use as file identifier (for exampe data_id='flare_time')
;                       default is 'my_data'
;   silent    - 0 or 1, if silent=1 no screen output, default is 0
;
; OUTPUTS:
;
; F(e)       - regularised photon spectrum, [photons/sec/cm^2/keV]
; dF         - 1 sigma error, [photons/sec/cm^2/keV]
; FWHM       - energy resolution of the regularised solution [keV]
;
;
; The results of the inversion are returned as a structure
;
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   Version 1, eduard(at)astro.gla.ac.uk, 23 November, 2005
;   returns a structure with resolution and errors
;-


FUNCTION FUNCTION_COUNTS2PHOTONS,e_c2N,eph2N,drm,counts,ecounts,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak,data_id=data_id,silent=silent
; function to calculate model-independent regularised photon flux spectrum

order=fcheck(order,0)
Npass=fcheck(Npass,30)
Guess=fcheck(Guess,.0)
reg_tweak=fcheck(reg_tweak,1.)
silent=fcheck(silent,0)
data_id=fcheck(data_id,'my_data')
time_inv=data_id

ee  =transpose(eph2N(0,*)+eph2N(1,*))/2.
;electron bin energy
e_c=transpose(e_c2N(0,*)+e_c2N(1,*))/2.
;count bin energy

de  =transpose(eph2N(1,*)-eph2N(0,*))
; electron energy bin width
de_c=transpose(e_c2N(1,*)-e_c2N(0,*))
; count energy bin width

; just to defined output files
sol_file ='ph'+time_inv+'_sol.dat'
sol_ps ='ph'+time_inv+'_sol.ps'
ps_file_err='ph'+time_inv+'_err.ps'
;output filenames: 'e'+time_interval+specifier

Matrix=transpose(drm)
;making drm suitable for inversion

ph_model = 1e3*ee^(-2)
; set as a decreasing function

cnt_model=Matrix##(ph_model)
;calculation the model of count flux spectrum

;p=fltarr(N_elements(counts),N_elements(counts))
;preconditioning

precond1=sqrt(cnt_model)
for i=0,N_elements(cnt_model)-1 do  Matrix(*,i)=Matrix(*,i)/precond1(i)
counts =counts/precond1
ecounts=ecounts/precond1

inv_make_constraint,L,ee,de,ph_model,order
print,'Constraint matrix   ................................. OK '

inv_gsvdcsq,Matrix,L,Alpha,Betta,U,V,W
print,'Generalized SV decomposition  .... ................. OK '

inv_reg_parameter,Alpha,Betta,U,W,counts,ecounts,Ph_model*Guess,reg_tweak,opt
print,'Regularisation parameter ................. OK '
;regularization parameter

inv_reg_solution,Alpha,Betta,U,W,counts,opt,ph_model*Guess,ph_reg
;solving reg solution
print,'Regularisation solution  ................. OK '

inv_reg_resolution,Alpha,Betta,opt,W,ee,de,FWHM

cnt_reg=Matrix##(ph_reg)

inv_residuals,cnt_reg,counts,ecounts,e_c,ps_file_err

inv_confidence_interval,ph_reg,counts,ecounts,Alpha,Betta,U,W,opt,ph_model,Guess,Npass,reg_sol_err
Print,'1 sigma confidence strip ...............................  OK'

counts=counts*precond1
ecounts=ecounts*precond1
cnt_reg=cnt_reg*precond1

inv_save_photons,ee,e_c,ph_reg,reg_sol_err,FWHM,ph_model,counts,ecounts,cnt_reg,sol_file,sol_ps

N_c=n_elements(e_c)
reg_solution={photons_reg:ph_reg(0:N_c-1),photons_err:reg_sol_err(0:N_c-1),eph_error:FWHM(0:N_c-1)}

return,reg_solution

PRINT,'Success. Results saved to file : ',sol_file

end

