;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;  inv_confidence_interval
;
;
; PURPOSE:
;   Calculates a confidence interval - 1 sigma uncertanty on
;   regularised solution
;
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
;  inv_confidence_interval,reg_sol,counts,ecounts,Alpha,Betta,U,W,opt,f_thin,Guess,Npass,reg_sol_err
;
; CALLS:
;
;   inv_reg_solution.pro
;
; INPUTS:
;
;   reg_sol - vector, regularised solution
;   counts  - vector, count flux
;   ecounts - vector, 1 sigma uncertanty on count flux,
;
;  Results of Generalised Singular Value Decomposition:
;   alpha  - diagonal elements of SA
;   betta  - diagonal elements of SB
;   U,W    - matrixes consisting of decomposition products
;
;   opt    - scalar, regularisation parameter
;   f_thin - vector, forward-fit results
;   Guess  - scalar (0 or 1.), use or not to use forward-fit in the constraint
;   Npass  - scalar, number of Monte Carlo simulations to produce confidence interval
;
; OPTIONAL INPUTS:
;   none
;
; OUTPUTS:
;
;  reg_sol_err- vector, 1 sigma uncertanty on regularised solution
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   eduard(at)astro.gla.ac.uk, 30 September, 2005
;-


pro inv_confidence_interval,reg_sol,counts,ecounts,Alpha,Betta,U,W,opt,f_thin,Guess,Npass,reg_sol_err
;calculating confidence strip

seed = 1001L
; Initial seed for a repeatable sequence

Strip_array=fltarr(Npass,N_elements(reg_sol))

for i=0, Npass-1 do begin
  rand_arr=(randomu(seed,N_elements(reg_sol))-0.5D0)*2.*ecounts
  ;print,'Confidence interval - pass # ',i,' out of ',Npass
  countX=counts+rand_arr
  inv_reg_solution,Alpha,Betta,U,W,countX,opt,f_thin*Guess,Reg_solX
  Strip_array(i,*)=reg_solX
end

reg_sol_err_plus=fltarr(N_elements(reg_sol))
reg_sol_err_minus=fltarr(N_elements(reg_sol))
for j=0, N_elements(reg_sol)-1 do begin
reg_sol_err_plus(j)=max((strip_array(*,j)-reg_sol(j)))
reg_sol_err_minus(j)=max(-(strip_array(*,j)-reg_sol(j)))
END

reg_sol_err=fltarr(N_elements(reg_sol))
for j=0, N_elements(reg_sol)-1 do reg_sol_err(j)=max(abs(strip_array(*,j)-reg_sol(j)))

;*****************************************************************************

end
