;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;
;   inv_counts2electrons
;
; PURPOSE:
;
;   Inversion of count spectrum using regularisation + GSVD method
;   to obtain electron distribution
;
;   Quick start with the inversion software
;   http://www.astro.gla.ac.uk/users/eduard/rhessi/inversion/
;
;  The bremsstrahlung produced photon flux spectrum I(eps)
; (photons/sec/cm^2/keV) at the Earth is given by the integral
;
;  I(eps)=1/(4*!Pi*1AU^2)*\int_{eps}^{infinity} S(eps,e)F(e)de
;
;  or in matrix form
;
;  I=S*F
;
;  where
;
;  F(e) = <V*n*Fe(e)>
;  F(e)  - density and volume weighted mean electron flux spectrum
;          [electrons/sec/cm^2/keV]
;
;  n - average number density of the surrounding plasma, [cm^-3]
;  V - average emitting volume, [cm^3]
;  S(eps,e) - differential bremsstrahlung cross-section, [cm^2/keV]
;  Fe(e) -  mean electron flux spectrum [electrons/sec/cm^2/keV]
;
;  The corresponding count flux spectrum in the X-ray detectors
;  C(eps) [counts/sec/cm^2/keV] can be written in matrix form
;
;  C = A*F, where A= DRM*S
;
;  We know C and A and want to find F. This problem is ill-conditioned
;  and regularision (additional constraints) should be applied.
;
;  The code finds the solution for the problem as a function of lambda
;
;  ||A*F-C||^2+lambda*||L*(F-F0)||^2=min
;
;   where
;   L  - constraint matrix
;   F0 - thin-target fit from SPEX/OSPEX
;   lambda - regularisation parameter found using descripancy principle
;
;
;   ||A*F(lambda)-C||=reg_tweak*||dC||
;
;   where
;   dC  - 1 sigma uncertanty on count flux, [counts/sec/cm^2/keV]
;
;   For details see:
;   Kontar et al,2004,Sol Phys,225,293-309
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2004SoPh..225..293K)
;   Kontar et al,2005,Sol Phys,226,317-325
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2005SoPh..226..317K)
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
; inv_counts2electrons,obj=obj,version=version,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak
;
;
; CALLS:
;
;
; INPUTS:
;
;
; OPTIONAL INPUTS:
;
;   obj       - OSPEX object name, if not set assumes old SPEX
;   version   - string, version of the inversion code (currently only one version exist)
;   order     - integer, regularization order. Can be 0 (default), 1, or 2.
;   Npass     - integer, number of random noise realisations for the confidence strip.
;                 Default is 30.
;   Guess     - integer, 1 or 0 (default) - use or NOT use forward-fit in the constraint
;   reg_tweak - float,  parameter to adjust the regularization parameter (default is 1).
;   ee_in     - 2D array of electron bin energies (2,Bin_Number) in keV
;               for example from 3 keV to 100 keV with 1 keV
;               ee_in=transpose([[findgen(100)+3.],[findgen(100)+4.]])
;
; OUTPUTS:
;
; F(e) =<VnFe>  - regularised electron spectrum, [10^50 electrons/sec/cm^2/keV]
; dF            - 1 sigma error, [10^50 electrons/sec/cm^2/keV]
; e             - middle point electron energy, keV
; FWHM          - energy resolution of the regularised solution [keV]
;
;
; The results of the inversion is the regularised electron flux
; saved to file with the name date of the observation
; Format: Y1Y1Y1Y1M1M1D1D1_H1H1M1M1S1S1_H2H2M2M2S2S2
; corresponding to the start 1 and end 2 of the time interval
;
; The results are also plotted into PS files.
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   Version 1, eduard(at)astro.gla.ac.uk, 23 July, 2004
;   30 September, 2005 - time interval added
;   31 October, 2005   - now works with OSPEX
;   15 january  2006   - added ee_in array option
;-


pro inv_counts2electrons,obj=obj,version=version,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak,ee_in=ee_in
;inversion using regularisation + GSVD method

z=1.2   ;z- Mean atomic number of the target plasma.
R=1.496e+13 ; 1AU distance in cm
R2pi4=((4.*!PI*R^2)/1e+28) ; flux coefficient

version=fcheck(version,'none')
order=fcheck(order,0)
Npass=fcheck(Npass,30)
Guess=fcheck(Guess,.0)
reg_tweak=fcheck(reg_tweak,1.)

verbose = keyword_set(verbose)

IF (version EQ 'anna') THEN BEGIN
MESSAGE, 'ANNA routine will strat here ....'
END

IF (n_elements(obj) EQ 0) THEN print,'Assuming SPEX ...............................'
IF (n_elements(obj) EQ 0) THEN inv_get_data_spex,e_edges,c_edges,counts,ecounts,drm,time_inv
; taking photon and error data from SPEX

IF (n_elements(obj) NE 0) THEN print,'Assuming OSPEX ...............................'
IF (n_elements(obj) NE 0) THEN inv_get_data_ospex,obj,e_edges,c_edges,counts,ecounts,drm,time_inv,efit_range
; taking photon and error data from OSPEX


ee =transpose(e_edges(0,*)+e_edges(1,*))/2.
ee_drm=ee
;electron bin energy
e_c=transpose(c_edges(0,*)+c_edges(1,*))/2.
;count bin energy

de  =transpose(e_edges(1,*)-e_edges(0,*))
; electron energy bin width
de_c=transpose(c_edges(1,*)-c_edges(0,*))
; count energy bin width

IF (n_elements(ee_in) NE 0) THEN BEGIN
ee_drm=ee
de_drm=de
ee =transpose(ee_in(0,*)+ee_in(1,*))/2.
de  =transpose(ee_in(1,*)-ee_in(0,*))
end

IF (n_elements(obj) NE 0) THEN inv_get_F_thin_ospex,obj,efit_range,ee,e_c,Fe_thin,I_thin
IF (n_elements(obj) EQ 0) THEN inv_get_F_thin_spex,ee,e_c,Fe_thin,I_thin
;takes thin target model parameters from SPEX/OSPEX

Fe_thin=Fe_thin*1e5
; it was devided by 511.before!!!
;Fe_thin=Fe_thin/R2pi4

sol_file ='e'+time_inv+'_sol.dat'
sol_ps ='e'+time_inv+'_sol.ps'
ps_file_err='e'+time_inv+'_err.ps'
;output filenames: 'e'+time_interval+specifier

; cross-section calculations
inv_cross_section,ee,ee_drm,z,cs_long,de
print,'Cross-section calculated .................. OK'
cs=cs_long/R2pi4

Matrix=drm#transpose(cs)
Matrix=transpose(Matrix)
Fit=Matrix##(Fe_thin)

;preconditioning
precond1=sqrt(fit)
for i=0,N_elements(fit)-1 do  Matrix(*,i)=Matrix(*,i)/precond1(i)
counts =counts/precond1
ecounts=ecounts/precond1

;PP=fltarr(n_elements(ee),n_elements(ee))
;for i=0,N_elements(ee)-1 do  PP(*,i)=Fe_thin

inv_make_constraint,L,ee,de,Fe_thin,order
print,'Constraint matrix   ................................. OK '

inv_gsvdcsq,Matrix,L,Alpha,Betta,U,V,W
print,'Generalized SV decomposition  .... ................. OK '

inv_reg_parameter,Alpha,Betta,U,W,counts,ecounts,Fe_thin*Guess,reg_tweak,opt
print,'Regularisation parameter ................. OK '
;regularization parameter

inv_reg_solution,Alpha,Betta,U,W,Counts,opt,Fe_thin*Guess,Reg_sol
;solving reg solution
print,'Regularisation solution  ................. OK '

inv_reg_resolution,Alpha,Betta,opt,w,ee,de,FWHM

ph_reg=Matrix##(reg_sol)

inv_residuals,ph_reg,counts,ecounts,e_c,ps_file_err

inv_confidence_interval,reg_sol,counts,ecounts,Alpha,Betta,U,W,opt,Fe_thin,Guess,Npass,reg_sol_err
Print,'1 sigma confidence strip ...............................  OK'

counts=counts*precond1
ecounts=ecounts*precond1
ph_reg=ph_reg*precond1

inv_save_electrons,ee,e_c,reg_sol,reg_sol_err,FWHM,Fe_thin,counts,ecounts,ph_reg,sol_file,sol_ps

PRINT,'Inversion completed. Results saved to file : ',sol_file

end
