;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;
;   inv_counts2photons
;
; PURPOSE:
;
;   Inversion of count spectrum using regularisation + GSVD method
;   to obtain model-independent photon spectrum
;
;   Quick start with the inversion software
;   http://www.astro.gla.ac.uk/users/eduard/rhessi/inversion/
;
;
;  The observed count flux spectrum in the X-ray detectors
;  C(eps) [counts/sec/cm^2/keV] can be written in matrix form
;
;  C = DRM*F
;
;  DRM - dynamic responce matrix of instrument
;
;  We know C and DRM and want to find a photon flux F  [counts/sec/cm^2/keV].
;  This problem is ill-conditioned
;  and regularision (additional constraints) should be applied.
;
;  The code finds the solution for the problem as a function of lambda
;
;  ||DRM*F-C||^2+lambda*||L*(F-F0)||^2=min
;
;   where
;
;   ||...|| - L2 norm
;   L       - constraint matrix
;   F0      - a model of photon spectrum from SPEX/OSPEX
;   lambda  - regularisation parameter found using descripancy principle
;
;
;   ||A*F(lambda)-C||=reg_tweak*||dC||
;
;   where
;   dC  - 1 sigma uncertanty on count flux, [counts/sec/cm^2/keV]
;
;   For details see:
;   Kontar et al,2004,Sol Phys,225,293-309
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2004SoPh..225..293K)
;   Kontar et al,2005,Sol Phys,226,317-325
;   (http://adsabs.harvard.edu/cgi-bin/nph-bib_query?bibcode=2005SoPh..226..317K)
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
; INV_COUNTS2PHOTONS,obj=obj,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak
;
; CALLS:
;
;
; INPUTS:
;   none

; OPTIONAL INPUTS:
;
;    obj      - OSPEX object name, if not provided assumes old SPEX
;   version   - string, version of the inversion code (currently only one version exist)
;   order     - integer, regularization order. Can be 0 (default), 1, or 2.
;   Npass     - integer, number of random noise realisations for the confidence strip.
;                 Default is 30.
;   Guess     - integer, 1 or 0 (default) - to use or NOT to use forward-fit in the constraint
;   reg_tweak - float,  parameter to adjust the regularization parameter (default is 1).
;
; OUTPUTS:
;
; F(e)       - regularised photon spectrum, [photons/sec/cm^2/keV]
; dF         - 1 sigma error, [photons/sec/cm^2/keV]
; FWHM       - energy resolution of the regularised solution [keV]
;
;
; The results of the inversion are returned as a structure
;
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   Version 1, eduard(at)astro.gla.ac.uk, 28 October, 2005
;   31/10/05, obj reference added
;-


PRO INV_COUNTS2PHOTONS,obj=obj,order=order,Npass=Npass,Guess=Guess,reg_tweak=reg_tweak
; function to calculate model-independent regularised photon flux spectrum

order=fcheck(order,0)
Npass=fcheck(Npass,30)
Guess=fcheck(Guess,.0)
reg_tweak=fcheck(reg_tweak,1.)

IF (n_elements(obj) EQ 0) THEN print,'Assuming SPEX ...............................'
IF (n_elements(obj) EQ 0) THEN inv_get_data_spex,eph2N,e_c2N,counts,ecounts,drm,time_inv,efit_range
; taking count and error data from SPEX

IF (n_elements(obj) NE 0) THEN print,'Assuming OSPEX ...............................'
IF (n_elements(obj) NE 0) THEN inv_get_data_ospex,obj,eph2N,e_c2N,counts,ecounts,drm,time_inv,efit_range
; taking count and error data from OSPEX

ee  =transpose(eph2N(0,*)+eph2N(1,*))/2.
;electron bin energy
e_c=transpose(e_c2N(0,*)+e_c2N(1,*))/2.
;count bin energy

de  =transpose(eph2N(1,*)-eph2N(0,*))
; electron energy bin width
de_c=transpose(e_c2N(1,*)-e_c2N(0,*))
; count energy bin width

IF (n_elements(obj) NE 0) THEN  BEGIN
ph_model1 = obj-> getdata(class='fit_function')
ph_model  = ph_model1(efit_range)
END

IF (n_elements(obj) EQ 0) THEN BEGIN
;inv_get_F_thin_spex,efit_range,ee,e_c,Fe_thin,I_thin
apar   =spex_current('apar',/incommon)
f_model=spex_current('f_model',/incommon)
ph_model= call_function(f_model, eph2N, apar)
;ph_model=ph_model1(efit_range)
;ph_model=ph_model1*de
END
;takes thin target model parameters from SPEX/OSPEX

sol_file ='ph'+time_inv+'_sol.dat'
sol_ps ='ph'+time_inv+'_sol.ps'
ps_file_err='ph'+time_inv+'_err.ps'
;output filenames: 'e'+time_interval+specifier

Matrix=transpose(drm)
;making drm suitable for inversion

cnt_model=Matrix##(ph_model)
;calculation the model of count flux spectrum

;p=fltarr(N_elements(counts),N_elements(counts))
;preconditioning

precond1=sqrt(cnt_model)
for i=0,N_elements(cnt_model)-1 do  Matrix(*,i)=Matrix(*,i)/precond1(i)
counts =counts/precond1
ecounts=ecounts/precond1

inv_make_constraint,L,ee,de,ph_model,order
print,'Constraint matrix   ................................. OK '

inv_gsvdcsq,Matrix,L,Alpha,Betta,U,V,W
print,'Generalized SV decomposition  .... ................. OK '

inv_reg_parameter,Alpha,Betta,U,W,counts,ecounts,Ph_model*Guess,reg_tweak,opt
print,'Regularisation parameter ................. OK '
;regularization parameter

inv_reg_solution,Alpha,Betta,U,W,counts,opt,ph_model*Guess,ph_reg
;solving reg solution
print,'Regularisation solution  ................. OK '

inv_reg_resolution,Alpha,Betta,opt,W,ee,de,FWHM

cnt_reg=Matrix##(ph_reg)

inv_residuals,cnt_reg,counts,ecounts,e_c,ps_file_err

inv_confidence_interval,ph_reg,counts,ecounts,Alpha,Betta,U,W,opt,ph_model,Guess,Npass,reg_sol_err
Print,'1 sigma confidence strip ...............................  OK'

counts=counts*precond1
ecounts=ecounts*precond1
cnt_reg=cnt_reg*precond1

inv_save_photons,ee,e_c,ph_reg,reg_sol_err,FWHM,ph_model,counts,ecounts,cnt_reg,sol_file,sol_ps

;N_c=n_elements(e_c)
;reg_solution={photons_reg:ph_reg(0:N_c-1),photons_err:reg_sol_err(0:N_c-1),eph_error:FWHM(0:N_c-1)}

;return,reg_solution

PRINT,'Success. Results saved to file : ',sol_file

end

