;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;   inv_get_data_spex
;
; PURPOSE:
;   Grabs all data from OSPEX for inversion
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;  inv_get_data_ospex
;
; CALLS:
;   none
;
; INPUTS:
;
;  obj - object name
;
; OPTIONAL INPUTS:
;   none
;
; OUTPUTS:
;
;   ee2N       - 2D array, electron energy bin edges
;   ec2N       - 2D array, count energy bin edges
;   counts     - count flux photons/kev/s/cm^2
;   Ephoton    - photon errors photons/kev/s/cm^2
;   drm_out    - spectral responce matrix
;                counts/sec/keV/cm^2 /(photons/sec/keV/cm^2)
;   time_inv   - string, time interval of the data
;   efit_range - array of indexes, electron energies to invert
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   Version 1, eduard(at)astro.gla.ac.uk, 30 July, 2005
;   30 September, 2005 - time interval added
;-

pro inv_get_data_ospex,obj,eph2N,e_c2N,counts,ecounts,drm_out,time_inv,efit_range
; gets data for the inversion

ct_energy=obj->getaxis(/edges_2,/ct_energy)
ph_energy=obj->getaxis(/edges_2,/ph_energy)

e_c =transpose(ct_energy(1,*)+ct_energy(0,*))/2.
de_c =transpose(ct_energy(1,*)-ct_energy(0,*))
e_ph=transpose(ph_energy(1,*)+ph_energy(0,*))/2.
de_ph=transpose(ph_energy(1,*)-ph_energy(0,*))

data_str = obj -> getdata(class='spex_fitint',spex_units='flux')
counts =data_str.data
ecounts=data_str.edata


drm_in = obj -> getdata(class='spex_drm')
; drm_in units [counts/sec/(photons/sec/cm^2/keV)]

area=obj->get(/spex_area)
for i=0, N_elements(e_ph)-1 do drm_in(*,i)=drm_in(*,i)/area/de_c

erange_i=obj -> getdata(class='spex_fitrange')

max_good =min(where((e_c GT 20.) and (ecounts*3. GT counts)))
; maximum good bin with count flux > 3 SIGMAS
;BUT above 20 keV (A3 max of count rate)

IF (max_good EQ -1) THEN max_good=N_elements(e_c)-1
; all points are good for the inversion

cfit_range=where((e_c  GE e_c(min(erange_i))) AND (e_c  LE e_c(max(erange_i)) ) AND (e_c LE e_c(max_good)))
efit_range=where((e_ph GE e_c(min(erange_i))) AND (e_ph LE e_c(max(cfit_range))*2.))


eph2N=Interpolate(ph_energy,[0,1],efit_range,/grid)
e_c2N=Interpolate(ct_energy,[0,1],cfit_range,/grid)

counts =counts(cfit_range)
ecounts=ecounts(cfit_range)

drm_out=Interpolate(drm_in,cfit_range,efit_range,/grid)

;******************************************************************
a = obj -> get(/spex_fit_time_interval)
if (n_elements(a) GT 2) THEN message,'ERROR: one time interval should be selected !'
time_inv1=time2file(round(a(0)),/sec)
time_inv2=time2file(round(a(1)),/sec)
time_inv =time_inv1+'_'+STRMID(time_inv2, 9, 6)
;******************************************************************

print,'Reading from',min(e_c2N),'...',max(e_c2N),'keV ........................   OK'
print,'Electron energies:',min(eph2N),'...',max(eph2N),'keV ........................   OK'

end
