;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;   inv_get_data_spex
;
; PURPOSE:
;   Grabs all data from SPEX for inversion
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;  inv_get_data_spex
;
; CALLS:
;   none
;
; INPUTS:
;   none
;
; OPTIONAL INPUTS:
;   none
;
; OUTPUTS:
;
;   ee       - photon energy bins 2D array
;   eph      - count  energy bins 2D array
;   Photon   - photon flux photons/kev/s/cm^2
;   Ephoton  - photon errors photons/kev/s/cm^2
;   drm_out  - Spectral Responce Matrix
;              counts/sec/cm^2/keV/(photons/sec/cm^2/keV)
;   time_inv - string, time interval of the data
;   efit_range - range of acceptable output energies
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   Version 1, eduard(at)astro.gla.ac.uk, 23 July, 2004
;   30 September, 2005 - time interval added
;-

pro inv_get_data_spex,ee2N,ec2N,counts,ecounts,drm_out,time_inv,efit_range


edges=spex_current('edges',/incommon)     ;edges(*,bins2get)
e_in =spex_current('e_in',/incommon)        ; =e_in(*,min(bins2get):n_elements(e_in)/2-1)
obsi =spex_current('obsi',/incommon)
backi=spex_current('backi',/incommon)
eobsi=spex_current('eobsi',/incommon)
ebacki=spex_current('ebacki',/incommon)

counts=obsi-backi   ;(bins2get)
ecounts=sqrt(eobsi^2+ebacki^2) ;(bins2get)

erange=spex_current('erange',/incommon)

e_c  =(edges(1,*)+edges(0,*))/2.
e_ph =(e_in(1,*)+e_in(0,*))/2.
de_ph =(e_in(1,*)-e_in(0,*))

drm_in=spex_current('drm',/incommon)
; responce matrix [ counts/sec/cm^2/keV/(photons/sec/cm^2)]

for i=0, N_elements(e_ph)-1 do drm_in(*,i)=drm_in(*,i)*de_ph(i)

max_good =min(where((e_c GT 20.) and (ecounts*3. GT counts)))
; maximum good bin with count flux > 3 SIGMAS
;BUT above 20 keV (A3 max of count rate)

IF (max_good EQ -1) THEN max_good=N_elements(e_c)-1
; all points are good for the inversion

cfit_range=where((e_c  GE erange(0)) AND (e_c  LE erange(1) ) AND (e_c LE e_c(max_good)))
efit_range=where((e_ph GE erange(0)) AND (e_ph LE e_c(max(cfit_range))*2.))

ee2N=Interpolate(E_in ,[0,1],efit_range,/grid)
ec2N=Interpolate(Edges,[0,1],cfit_range,/grid)

counts =counts(cfit_range)
ecounts=ecounts(cfit_range)

drm_out=Interpolate(drm_in,cfit_range,efit_range,/grid)

;******************************************************************

iselect=spex_current('iselect',/incommon)
ut=spex_current('ut',/incommon)
xselect1=spex_current('xselect',/incommon)

time_inv1=time2file(getutbase(0)+ROUND(ut(0,iselect(0,0))),/sec)
time_inv2=time2file(getutbase(0)+ROUND(ut(1,iselect(1,0))),/sec)
time_inv =time_inv1+'_'+STRMID(time_inv2, 9, 6)

;******************************************************************

print,'Reading from',min(ec2N),'...',max(ec2N),'keV ........................   OK'
print,'Electron energies:',min(ee2N),'...',max(ee2N),'keV ........................   OK'

end
