;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;   inv_reg_parameter
;
; PURPOSE:
;   to compute regularization parameter
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
;  inv_reg_parameter,sigmaA,SigmaB,U,W,Data,Err,Fbar_fit,reg_tweak,opt
;
; CALLS:
;   none
;
; INPUTS:
;  SigmaA - vector, generalised singular values
;  SigmaB - vector, generalised singular values
;  U      - matrix, GSVD matrix
;  W      - matrix, GSVD matrix
;  Data   - vector, containing count flux
;  Err    - vector, uncertanty on count flux
;  Fbar_fit-vector, forward-fit for the electron distribution
; reg_tweak-scalar, parameter to adjusting regularization
;
; OPTIONAL INPUTS:
;   none
;
; OUTPUTS:
;    opt -  regularization parameter
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   eduard(at)astro.gla.ac.uk, 23 May, 2005
;   16- Sept-2005: now plots to display and eps files
;-


pro inv_reg_parameter,sigmaA,SigmaB,U,W,Data,Err,Fbar_fit,reg_tweak,opt
;calculates regularisation parameter

 Nmu=9900

 Ndata=N_elements(Data)
 Nreg =N_elements(SigmaA)

 arg=dblarr(Nreg,Nmu)
 arg2=dblarr(Nreg,Nmu)
 discr=dblarr(Nmu)

 maxx=max(SigmaA/SigmaB)
 minx=max(SigmaA/SigmaB)*1e-10

 step=(alog(maxx)-alog(minx))/(Nmu-1.)
 mu=exp(findgen(Nmu)*step)*minx

 omega=invert(w)##Fbar_fit

 for k=0,Ndata-1 do begin
   coef=data##u[k,*]-SigmaA(k)*omega(k)
   for i=0,Nmu-1 do begin
   arg[k,i]=(mu[i]*SigmaB(k)*SigmaB(k)*coef/(SigmaA(k)*SigmaA(k)+mu[i]*SigmaB(k)*SigmaB(k)))^2
   ;arg2[k,i]=(mu[i]*SigmaB(k)*SigmaB(k)*coef/(SigmaA(k)*SigmaA(k)+mu[i]*SigmaB(k)*SigmaB(k)))^2
   end
   arg2[k,*]=sqrt(arg[k,*])/Err(k)
 end

discr=total(arg,1)-total(err*err)*reg_tweak
;discr=total(arg2*arg2,1)-reg_tweak*float(Ndata)

minimum=min(abs(discr),Min_index)
opt=mu[Min_index]

SV=abs(sigmaA)/abs(SigmaA^2+opt*SigmaB^2)
Data_U=abs(data##u)
C=data_u*SV

;****************************************************************************************
window,1,xsize=600,ysize=600,title='Regularization parameter and Picard condition'
!P.MULTI=[0,1,2]
plot_oo, mu, abs(discr), xrange=[minx,maxx],psym=1,Title='Tikhonov regularization',$
ytitle='Descripancy',xtitle='Regularization parameter, !4k!3'
plot,findgen(Nreg)+1.,C, xrange=[1,Ndata],xstyle=1,/xlog,Psym=10,$
Title='Picard Condition',xtitle='Singular Value number',/ylog
oplot,findgen(Nreg)+1.,data_u,line=1
oplot,findgen(Nreg)+1.,SV,line=2
!P.MULTI=0
;****************************************************************************************

current_plot=!D.name
Set_plot, 'PS'
Device,filename='picard.eps', xsize=18,ysize=18,ENCAPSULATED=1
!P.MULTI=[0,1,2]
plot_oo, mu, abs(discr), xrange=[minx,maxx],psym=1,Title='Tikhonov regularization',$
ytitle='Descripancy',xtitle='Regularization parameter, !4k!3'

plot,findgen(Nreg)+1.,C,xrange=[1,Ndata],xstyle=1,/xlog,Psym=10,$
Title='Picard Condition',xtitle='Singular Value number',/ylog,yrange=[C(Ndata-1),max(C)],ystyle=1
oplot,findgen(Nreg)+1.,data_u,line=1
oplot,findgen(Nreg)+1.,SV,line=2
!P.MULTI=0
DEVICE, /CLOSE
SET_PLOT, current_plot

PRINT, 'Regularization parameter (discrepancy): ', opt
;****************************************************************************************

end

