;+
; PROJECT:
;   RHESSI/INVERSION
;
; NAME:
;   inv_reg_resolution
;
; PURPOSE:
;   Calculates resolution of regularised solution
;
; CATEGORY:
;   HESSI, Spectra, Inversion
;
; CALLING SEQUENCE:
;
;  inv_reg_resolution,Alpha,Betta,opt,w,ee,de,FWHM
;
; CALLS:
;   none
;
; INPUTS:
;
;  Alpha  - vector, generalised singular values
;  Betta  - vector, generalised singular values
;  opt    - regularisation parameter
;  w      - matrix, generalised singular value decomposition
;  ee     - electron bin average energy, keV
;  de     - electron bin width, keV
;
; OPTIONAL INPUTS:
;   none
;
; OUTPUTS:
;   FWHM -  resolution in keV defined as full width at half maximum
;           of the resolution matrix
;
; OPTIONAL OUTPUTS:
;   none
;
; KEYWORDS:
;   none
;
; COMMON BLOCKS:
;   none
;
; SIDE EFFECTS:
;
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;   eduard(at)astro.gla.ac.uk, 23 Sept, 2005
;-

pro inv_reg_resolution,Alpha,Betta,opt,w,ee,de,FWHM
;calculates resolution of the regularised solution
; resolution is defined as FWHM of the resolution matrix

M=n_elements(Alpha)

Filter=fltarr(M,M)
For i=0, M-1 do Filter(i,i)=alpha(i)^2/(alpha(i)^2+opt*betta(i)^2)

R =w##(Filter##invert(w))

FWHM=fltarr(M)

For i=0, M-1 do BEGIN
HM_index=where(R(i,*) GT max(R(i,*))/2.)
IF (N_elements(HM_index) EQ 1) THEN HM_index=i
HM=min(ee(HM_index))
FWHM(i)=de(i)*((ee(i)-HM) LT de(i))+(ee(i)-HM)*((ee(i)-HM) GE de(i))
end

FWHM=FWHM*2.

end



